use bevy::prelude::*;

#[derive(Component, Deref, DerefMut, Debug, Default)]
pub struct Velocity(pub Vec2);

#[derive(Component, Deref, DerefMut, Debug, Default)]
pub struct Heading(pub Vec2);

#[derive(Component, Deref, DerefMut, Debug, Default)]
pub struct Speed(pub f32);

fn update_position(mut query: Query<(&Velocity, &mut Transform)>, time: Res<Time>) {
    for (velocity, mut transform) in query.iter_mut() {
        transform.translation += (velocity.0 * time.delta_seconds()).extend(0.);
    }
}

fn mover_movement(mut query: Query<(&mut Velocity, &Speed, &Heading)>, input: Res<Input<KeyCode>>) {
    let (mut velocity, speed, heading) = query.single_mut();
    **velocity = heading.normalize_or_zero() * (**speed);
}

#[derive(Bundle, Debug, Default)]
pub struct Mover {
    pub velocity: Velocity,
    pub heading: Heading,
    pub speed: Speed,
}

pub struct Movement;

impl Plugin for Movement {
    fn build(&self, app: &mut App) {
        // TODO: these systems probably needs ordering?
        app.add_systems(Update, (mover_movement, update_position));
    }
}
