use bevy::prelude::*;

use crate::{
    assets::AssetLoader,
    movement::{Heading, Mover, Speed, Velocity},
};

const PLAYER_SPEED: f32 = 500.;

#[derive(Component, Debug)]
pub struct Player;

impl Plugin for Player {
    fn build(&self, app: &mut App) {
        app.add_systems(PostStartup, spawn_player)
            .add_systems(Update, player_movement);
    }
}

fn spawn_player(mut commands: Commands, assets: Res<AssetLoader>) {
    commands.spawn((
        Player,
        SpriteSheetBundle {
            texture_atlas: assets.sprites.player.clone(),
            sprite: TextureAtlasSprite::new(0),
            transform: Transform::from_scale(Vec3::splat(3.0)),
            ..Default::default()
        },
        Mover {
            velocity: Velocity(Vec2::ZERO),
            heading: Heading(Vec2::ZERO),
            speed: Speed(PLAYER_SPEED),
        },
    ));
}

fn player_movement(mut query: Query<&mut Heading, With<Player>>, input: Res<Input<KeyCode>>) {
    let mut heading = query.single_mut();
    **heading = Vec2::ZERO;
    for key in input.get_pressed() {
        match key {
            KeyCode::W | KeyCode::Up => {
                **heading += Vec2::Y;
            }
            KeyCode::S | KeyCode::Down => {
                **heading -= Vec2::Y;
            }
            KeyCode::A | KeyCode::Left => {
                **heading -= Vec2::X;
            }
            KeyCode::D | KeyCode::Right => {
                **heading += Vec2::X;
            }
            _ => {}
        }
    }
}
